@@ -41,7 +41,7 @@ use SNMP::Info;
 
 use vars qw/$VERSION %MIBS %FUNCS %GLOBALS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.34';
 
 %MIBS = (
     'CISCO-VTP-MIB'                       => 'vtpVlanName',
@@ -149,6 +149,7 @@ sub i_vlan {
 
     my $port_vlan      = $vtp->vtp_trunk_native($partial)   || {};
     my $i_vlan         = $vtp->i_vlan2($partial)            || {};
+    my $trunk_dyn      = $vtp->vtp_trunk_dyn($partial)      || {};
     my $trunk_dyn_stat = $vtp->vtp_trunk_dyn_stat($partial) || {};
 
     my %i_vlans;
@@ -165,15 +166,23 @@ sub i_vlan {
     foreach my $port ( keys %$port_vlan ) {
         my $vlan = $port_vlan->{$port};
         next unless defined $vlan;
+
+        # ports up and trunking should have a trunking status
         my $stat = $trunk_dyn_stat->{$port};
-        if ( defined $stat and $stat =~ /^trunking/ ) {
+
+        # vtp_trunk_dyn_stat is not useful for down ports
+        # so we use vtp_trunk_dyn to see if trunking is set
+        my $dyn = $trunk_dyn->{$port};
+        
+        if (($stat and $stat =~ /^trunking/ )
+            or ($dyn and (($dyn eq 'on') or ($dyn eq 'onNoNegotiate'))))
+        {
             $i_vlans{$port} = $vlan;
         }
     }
 
     # Check in CISCO-VLAN-IFTABLE-RELATION-MIB
-    # Is this only for Aironet???  If so, it needs
-    # to be removed from this class
+    # Used for traditional Cisco Routers and Aironet
 
     my $v_cvi_if = $vtp->v_cvi_if();
     if ( defined $v_cvi_if ) {
@@ -191,6 +200,22 @@ sub i_vlan {
     return \%i_vlans;
 }
 
+sub i_untagged {
+    my $vtp = shift;
+    my ( $ifindex ) = @_;
+
+    # cannot defer to i_vlan - vtp_trunk_dyn_stat is not useful for down ports
+    # so we use vtp_trunk_dyn as a hint to use i_pvid
+
+    my $trunking = eval { $vtp->vtp_trunk_dyn($ifindex)->{$ifindex} };
+    if ($trunking and (($trunking eq 'on') or ($trunking eq 'onNoNegotiate'))) {
+        return $vtp->i_pvid(@_);
+    }
+    else {
+        return $vtp->i_vlan(@_);
+    }
+}
+
 sub i_vlan_membership {
     my $vtp     = shift;
     my $partial = shift;
@@ -202,6 +227,8 @@ sub i_vlan_membership {
     my $vtp_vlans      = $vtp->v_state();
     my $i_vlan         = $vtp->i_vlan2($partial)            || {};
     my $trunk_dyn_stat = $vtp->vtp_trunk_dyn_stat($partial) || {};
+    my $trunk_dyn      = $vtp->vtp_trunk_dyn($partial)      || {};
+    my $i_voice_vlan   = $vtp->i_voice_vlan($partial)       || {};
 
     my $i_vlan_membership = {};
 
@@ -209,14 +236,24 @@ sub i_vlan_membership {
     foreach my $port ( keys %$i_vlan ) {
         my $vlan = $i_vlan->{$port};
         next unless defined $vlan;
-        my $stat = $trunk_dyn_stat->{$port};
-        if ( defined $stat and $stat =~ /notTrunking/ ) {
+        my $dyn = $trunk_dyn->{$port};
+        unless ($dyn and (($dyn eq 'on') or ($dyn eq 'onNoNegotiate'))) {
+            push( @{ $i_vlan_membership->{$port} }, $vlan );
+        }
+    }
+
+    # Get voice VLANs
+    foreach my $port ( keys %$i_voice_vlan ) {
+        my $vlan = $i_voice_vlan->{$port};
+        next unless defined $vlan;
+        next unless ($vlan =~ m/[[:digit:]]+/ and $vlan < 4095);
+        my $dyn = $trunk_dyn->{$port};
+        unless ($dyn and (($dyn eq 'on') or ($dyn eq 'onNoNegotiate'))) {
             push( @{ $i_vlan_membership->{$port} }, $vlan );
         }
     }
 
     # Get trunk ports
-
     my %oper_vlans;
     foreach my $iid ( keys %$vtp_vlans ) {
         my $vlan    = 0;
@@ -233,7 +270,9 @@ sub i_vlan_membership {
 
     foreach my $port ( keys %$ports_vlans ) {
         my $stat = $trunk_dyn_stat->{$port};
-        if ( defined $stat and $stat =~ /^trunking/ ) {
+        my $dyn = $trunk_dyn->{$port};
+        if (($stat and $stat =~ /^trunking/ )
+            or ($dyn and (($dyn eq 'on') or ($dyn eq 'onNoNegotiate')))) {
             my $k     = 0;
             my $list1 = $ports_vlans->{$port} || '0';
             my $list2 = $ports_vlans_2k->{$port} || '0';
@@ -244,6 +283,7 @@ sub i_vlan_membership {
                 next unless $list;
                 my $vlanlist = [ split( //, unpack( "B*", $list ) ) ];
                 foreach my $vlan ( keys %oper_vlans ) {
+                    next if (($vlan < $offset) or ($vlan - $offset > 1024));
                     push( @{ $i_vlan_membership->{$port} }, $vlan )
                         if ( @$vlanlist[ $vlan - $offset ] );
                 }
@@ -254,6 +294,20 @@ sub i_vlan_membership {
     return $i_vlan_membership;
 }
 
+sub i_vlan_membership_untagged {
+    my $vtp  = shift;
+    my $partial = shift;
+
+    my $vlans = $vtp->i_vlan($partial);
+    my $i_vlan_membership = {};
+    foreach my $port (keys %$vlans) {
+        my $vlan = $vlans->{$port};
+        push( @{ $i_vlan_membership->{$port} }, $vlan );
+    }
+    
+    return $i_vlan_membership;
+}
+
 sub set_i_pvid {
     my $vtp = shift;
     my ( $vlan_id, $ifindex ) = @_;
@@ -305,6 +359,22 @@ sub set_i_vlan {
     return;
 }
 
+sub set_i_untagged {
+    my $vtp = shift;
+    my ( $vlan_id, $ifindex ) = @_;
+
+    # cannot defer to i_vlan - vtp_trunk_dyn_stat is not useful for down ports
+    # so we use vtp_trunk_dyn as a hint to use i_pvid
+
+    my $trunking = eval { $vtp->vtp_trunk_dyn($ifindex)->{$ifindex} };
+    if ($trunking and (($trunking eq 'on') or ($trunking eq 'onNoNegotiate'))) {
+        return $vtp->set_i_pvid(@_);
+    }
+    else {
+        return $vtp->set_i_vlan(@_);
+    }
+}
+
 sub set_add_i_vlan_tagged {
     my $vtp = shift;
     my ( $vlan_id, $ifindex ) = @_;
@@ -539,6 +609,10 @@ Your device will only implement a subset of these methods.
 Returns a mapping between C<ifIndex> and assigned VLAN ID for access ports
 and the default VLAN ID for trunk ports.
 
+=item $vtp->i_untagged()
+
+An alias for C<i_vlan>.
+
 =item $vtp->i_vlan_membership()
 
 Returns reference to hash of arrays: key = C<ifIndex>, value = array of VLAN
@@ -554,6 +628,12 @@ IDs.  These are the VLANs which are members of enabled VLAN list for the port.
     print "Port: $port VLAN: $vlan\n";
   }
 
+=item $vtp->i_vlan_membership_untagged()
+
+Returns reference to hash of arrays: key = C<ifIndex>, value = array of VLAN
+IDs.  These are the VLANs which are members of the untagged egress list for
+the port.
+
 =back
 
 =head2 VLAN Table (C<CISCO-VTP-MIB::vtpVlanTable>)
@@ -834,6 +914,12 @@ port C<ifIndex>.  This method should only be used on trunk ports.
   $vtp->set_i_pvid('2', $if_map{'FastEthernet0/1'}) 
     or die "Couldn't change port default VLAN. ",$vtp->error(1);
 
+=item $vtp->set_i_untagged ( vlan, ifIndex )
+
+This method attempts to work out whether the port referenced by ifIndex is
+trunking, and if so will return the value of C<set_i_pvid>. Otherwise, the
+value of C<set_i_vlan> is returned.
+
 =item $vtp->set_add_i_vlan_tagged ( vlan, ifIndex )
 
 Adds the VLAN to the enabled VLANs list of the port, must be supplied with the
