@@ -39,7 +39,7 @@ use SNMP::Info::Layer2;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '2.05';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::Layer2::MIBS,
@@ -324,6 +324,13 @@ sub bp_index {
     return \%bp_index;
 }
 
+sub fw_mac {
+    my $s3000 = shift;
+    my $partial   = shift;
+
+    return $s3000->SUPER::fw_mac($partial);
+}
+
 sub fw_port {
     my $s3000   = shift;
     my $partial = shift;
