@@ -46,12 +46,13 @@ use SNMP::Info::Layer3;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,    %SNMP::Info::LLDP::MIBS,
     %SNMP::Info::RapidCity::MIBS, %SNMP::Info::NortelStack::MIBS,
     %SNMP::Info::SONMP::MIBS,
+    'BAY-STACK-PETH-EXT-MIB' => 'bspePethPsePortExtMeasuredPower',
 );
 
 %GLOBALS = (
@@ -64,6 +65,7 @@ $VERSION = '2.06';
     %SNMP::Info::Layer3::FUNCS,    %SNMP::Info::LLDP::FUNCS,
     %SNMP::Info::RapidCity::FUNCS, %SNMP::Info::NortelStack::FUNCS,
     %SNMP::Info::SONMP::FUNCS,
+    'peth_port_power' => 'bspePethPsePortExtMeasuredPower',
 );
 
 # 450's report full duplex as speed = 20mbps?!
@@ -79,27 +81,20 @@ $SNMP::Info::SPEED_MAP{2_000_000_000} = '1.0 Gbps';
 
 sub os {
     my $baystack = shift;
-    my $descr    = $baystack->description();
-    my $model    = $baystack->model();
+    my $descr    = $baystack->description() || "";
+    my $model    = $baystack->model() || "";
 
-    if ( ( defined $descr and $descr =~ /Business Ethernet Switch.*SW:v/i ) )
-    {
+    if ( $descr =~ /Business Ethernet Switch.*SW:v/i ) {
         return 'bes';
     }
-    if (
-        (
-           (defined $model and $model =~ /(420|425|BPS)/ )
-        and
-           (defined $descr and $descr =~ m/SW:v[1-2]/i )
-        )
-        or
-        (
-            (defined $model and $model =~ /(410|450|380)/ )
-        )
-       )
+    if (   ( ( $model =~ /(420|425|BPS)/ ) and ( $descr =~ m/SW:v[1-2]/i ) )
+        or ( ( $model =~ /(410|450|380)/ ) ) )
     {
         return 'baystack';
     }
+    if ( $model =~ /VSP/ ) {
+        return 'vsp';
+    }
 
     return 'boss';
 }
@@ -126,7 +121,7 @@ sub os_bin {
 }
 
 sub vendor {
-    return 'nortel';
+    return 'avaya';
 }
 
 sub model {
@@ -141,9 +136,12 @@ sub model {
     return '303' if ( defined $descr and $descr =~ /\D303\D/ );
     return '304' if ( defined $descr and $descr =~ /\D304\D/ );
     return 'BPS' if ( $model =~ /BPS2000/i );
-    return $2
-        if ( $model
-        =~ /(ES|ERS|BayStack|EthernetRoutingSwitch|EthernetSwitch)-?(\d+)/ );
+    
+    # Pull sreg- from all
+    $model =~ s/^sreg-//;
+    # Strip ES/ERS/BayStack etc. from those families
+    $model =~ s/^(E(R)?S|BayStack|Ethernet(Routing)?Switch)-?//;
+    $model =~ s/-ethSwitchNMM//;
 
     return $model;
 }
@@ -162,7 +160,7 @@ sub interfaces {
         next unless defined $index;
 
         # Ignore cascade ports
-        next if $index > 513;
+        next if $index > $index_factor * 8;
 
         my $port = ( $index % $index_factor );
         my $slot = ( int( $index / $index_factor ) ) + $slot_offset;
@@ -214,7 +212,7 @@ sub i_name {
 
 sub index_factor {
     my $baystack = shift;
-    my $model    = $baystack->model();
+    my $model    = $baystack->model() || "";
     my $os       = $baystack->os();
     my $os_ver   = $baystack->os_ver();
     my $op_mode  = $baystack->ns_op_mode();
@@ -228,142 +226,16 @@ sub index_factor {
 
     my $index_factor = 32;
     $index_factor = 64
-        if ( ( defined $model and $model =~ /(470)/ )
+        if ( ( $model =~ /(470)/ )
         or ( $os =~ m/(boss|bes)/ ) and ( $op_mode eq 'pure' ) );
     $index_factor = 128
-        if ( ( defined $model and $model =~ /(5[56]\d\d)/ )
+        if ( ( $model =~ /(5[56]\d\d)|VSP/ )
         and ( $os_ver >= 6 ) );
 
     return $index_factor;
 }
 
-#  Use SONMP and/or LLDP
 
-sub hasCDP {
-    my $baystack = shift;
-
-    return $baystack->hasLLDP() || $baystack->SUPER::hasCDP();
-}
-
-sub c_ip {
-    my $baystack = shift;
-    my $partial  = shift;
-
-    my $cdp  = $baystack->SUPER::c_ip($partial) || {};
-    my $lldp = $baystack->lldp_ip($partial)     || {};
-
-    my %c_ip;
-    foreach my $iid ( keys %$cdp ) {
-        my $ip = $cdp->{$iid};
-        next unless defined $ip;
-
-        $c_ip{$iid} = $ip;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $ip = $lldp->{$iid};
-        next unless defined $ip;
-
-        $c_ip{$iid} = $ip;
-    }
-    return \%c_ip;
-}
-
-sub c_if {
-    my $baystack = shift;
-    my $partial  = shift;
-
-    my $lldp = $baystack->lldp_if($partial)     || {};
-    my $cdp  = $baystack->SUPER::c_if($partial) || {};
-
-    my %c_if;
-    foreach my $iid ( keys %$cdp ) {
-        my $if = $cdp->{$iid};
-        next unless defined $if;
-
-        $c_if{$iid} = $if;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $if = $lldp->{$iid};
-        next unless defined $if;
-
-        $c_if{$iid} = $if;
-    }
-    return \%c_if;
-}
-
-sub c_port {
-    my $baystack = shift;
-    my $partial  = shift;
-
-    my $lldp = $baystack->lldp_port($partial)     || {};
-    my $cdp  = $baystack->SUPER::c_port($partial) || {};
-
-    my %c_port;
-    foreach my $iid ( keys %$cdp ) {
-        my $port = $cdp->{$iid};
-        next unless defined $port;
-
-        $c_port{$iid} = $port;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $port = $lldp->{$iid};
-        next unless defined $port;
-
-        $c_port{$iid} = $port;
-    }
-    return \%c_port;
-}
-
-sub c_id {
-    my $baystack = shift;
-    my $partial  = shift;
-
-    my $lldp = $baystack->lldp_id($partial)     || {};
-    my $cdp  = $baystack->SUPER::c_id($partial) || {};
-
-    my %c_id;
-    foreach my $iid ( keys %$cdp ) {
-        my $id = $cdp->{$iid};
-        next unless defined $id;
-
-        $c_id{$iid} = $id;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $id = $lldp->{$iid};
-        next unless defined $id;
-
-        $c_id{$iid} = $id;
-    }
-    return \%c_id;
-}
-
-sub c_platform {
-    my $baystack = shift;
-    my $partial  = shift;
-
-    my $lldp = $baystack->lldp_rem_sysdesc($partial)  || {};
-    my $cdp  = $baystack->SUPER::c_platform($partial) || {};
-
-    my %c_platform;
-    foreach my $iid ( keys %$cdp ) {
-        my $platform = $cdp->{$iid};
-        next unless defined $platform;
-
-        $c_platform{$iid} = $platform;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $platform = $lldp->{$iid};
-        next unless defined $platform;
-
-        $c_platform{$iid} = $platform;
-    }
-    return \%c_platform;
-}
 
 # Newer devices support ENTITY-MIB, use if available otherwise use proprietary
 # methods.
@@ -452,14 +324,85 @@ sub e_vendor {
     return $stack->SUPER::e_vendor($partial) || $stack->ns_e_vendor($partial);
 }
 
+# fix for stack of switches without POE on module 1
+# https://sourceforge.net/tracker/?func=detail&aid=3317739&group_id=70362&atid=527529
+sub peth_port_ifindex {
+    my $stack = shift;
+    my $partial = shift;
+
+    my %peth_port_ifindex = ();
+    my $poe_port_st = $stack->peth_port_status($partial);
+    my $if_index = $stack->interfaces($partial);
+
+    foreach my $i (keys %$if_index) {
+        next unless defined $poe_port_st->{$if_index->{$i}};
+        $peth_port_ifindex{$if_index->{$i}} = $i;
+    }
+    return \%peth_port_ifindex;
+}
+
+# Currently only ERS 4800 v5.8+ support the rcBridgeSpbmMacTable 
+# which holds the FDB for a SPBM edge deployment.
+#
+# Q-BRIDGE still holds some entries when the rcBridgeSpbmMacTable is in use
+# so we merge hash entries.
+
+sub fw_mac {
+    my $rapidcity = shift;
+
+    my $qb = $rapidcity->SUPER::fw_mac() || {};
+    my $spbm = $rapidcity->rc_spbm_fw_mac() || {};
+    my $fw_mac = { %$qb, %$spbm };
+    
+    return $fw_mac;
+}
+
+sub fw_port {
+    my $rapidcity = shift;
+
+    my $qb = $rapidcity->SUPER::fw_port() || {};
+    my $spbm = $rapidcity->rc_spbm_fw_port() || {};
+    my $fw_port = { %$qb, %$spbm };
+    
+    return $fw_port;
+}
+
+sub fw_status {
+    my $rapidcity = shift;
+
+    my $qb = $rapidcity->SUPER::fw_status() || {};    
+    my $spbm = $rapidcity->rc_spbm_fw_status() || {};
+    my $fw_status = { %$qb, %$spbm };
+    
+    return $fw_status;
+}
+
+sub qb_fw_vlan {
+    my $rapidcity = shift;
+
+    my $qb = $rapidcity->SUPER::qb_fw_vlan() || {};
+    my $spbm = $rapidcity->rc_spbm_fw_vlan() || {};
+    my $qb_fw_vlan = { %$qb, %$spbm };
+    
+    return $qb_fw_vlan;
+}
+
+# Baystack uses S5-AGENT-MIB (loaded in NortelStack) versus RAPID-CITY
+sub stp_ver {
+    my $rapidcity = shift;
+
+    return $rapidcity->s5AgSysSpanningTreeOperMode()
+      || $rapidcity->SUPER::stp_ver();
+}
+
 1;
 
 __END__
 
 =head1 NAME
 
-SNMP::Info::Layer2::Baystack - SNMP Interface to Nortel Ethernet (Baystack)
-Switches
+SNMP::Info::Layer2::Baystack - SNMP Interface to Avaya Ethernet Switch
+(Baystack) and VSP 7000 series switches
 
 =head1 AUTHOR
 
@@ -482,8 +425,9 @@ Eric Miller
 
 =head1 DESCRIPTION
 
-Provides abstraction to the configuration information obtainable from a Nortel 
-Ethernet Switch (Baystack) through SNMP. 
+Provides abstraction to the configuration information obtainable from an
+Avaya Ethernet Switch (formerly Nortel/Bay Baystack) and VSP 7000 series
+through SNMP. 
 
 For speed or debugging purposes you can call the subclass directly, but not
 after determining a more specific class using the method above. 
@@ -510,6 +454,8 @@ my $baystack = new SNMP::Info::Layer2::Baystack(...);
 
 =over
 
+=item F<BAY-STACK-PETH-EXT-MIBB>
+
 =back
 
 =head2 Inherited MIBs
@@ -532,7 +478,7 @@ These are methods that return scalar value from SNMP
 
 =item $baystack->vendor()
 
-Returns 'nortel'
+Returns 'avaya'
 
 =item $baystack->model()
 
@@ -552,6 +498,14 @@ Returns 'baystack' or 'boss' depending on software version.
 
 Returns the firmware version extracted from C<sysDescr>.
 
+=item $baystack->stp_ver()
+
+Returns the particular STP version running on this device.  
+
+Values: C<nortelStpg>, C<pvst>, C<rstp>, C<mstp>, C<ieee8021d>
+
+(C<s5AgSysSpanningTreeOperMode>)
+
 =back
 
 =head2 Overrides
@@ -628,6 +582,16 @@ revisions of Baystack firmware report all zeros for each port mac.
 Crosses C<ifName> with C<ifAlias> and returns the human set port name if
 exists.
 
+=item $baystack->peth_port_ifindex()
+
+Maps the C<pethPsePortTable> to C<ifIndex> by way of the F<ENTITY-MIB>.
+
+=item $baystack->peth_port_power()
+
+Power supplied by PoE ports, in milliwatts
+
+(C<bspePethPsePortExtMeasuredPower>)
+
 =back
 
 =head2 F<ENTITY-MIB> Information
@@ -698,51 +662,29 @@ ns_e_vendor().
 
 =back
 
-=head2 Topology information
+=head2 Layer 2 Forwarding Database
 
-Based upon the software version devices may support SynOptics Network
-Management Protocol (SONMP) and Link Layer Discovery Protocol (LLDP). These
-methods will query both and return the combination of all information. As a
-result, there may be identical topology information returned from the two
-protocols causing duplicate entries.  It is the calling program's
-responsibility to identify any duplicate entries and remove duplicates if
-necessary.
+These methods try to obtain the layer 2 forwarding database entries via the
+normal bridge methods as well as SPBM entries via rapid city methods.
 
 =over
 
-=item $baystack->hasCDP()
+=item $baystack->fw_mac()
 
-Returns true if the device is running either SONMP or LLDP.
+Returns reference to hash of forwarding table MAC Addresses
 
-=item $baystack->c_if()
+=item $baystack->fw_port()
 
-Returns reference to hash.  Key: iid Value: local device port (interfaces)
+Returns reference to hash of forwarding table entries port interface
+identifier (iid)
 
-=item $baystack->c_ip()
+=item $baystack->qb_fw_vlan()
 
-Returns reference to hash.  Key: iid Value: remote IPv4 address
+Returns reference to hash of forwarding table entries VLAN ID
 
-If multiple entries exist with the same local port, c_if(), with the same IPv4
-address, c_ip(), it may be a duplicate entry.
+=item $baystack->fw_status()
 
-If multiple entries exist with the same local port, c_if(), with different
-IPv4 addresses, c_ip(), there is either a non-SONMP/LLDP device in between two or
-more devices or multiple devices which are not directly connected.  
-
-Use the data from the Layer2 Topology Table below to dig deeper.
-
-=item $baystack->c_port()
-
-Returns reference to hash. Key: iid Value: remote port (interfaces)
-
-=item $baystack->c_id()
-
-Returns reference to hash. Key: iid Value: string value used to identify the
-chassis component associated with the remote system.
-
-=item $baystack->c_platform()
-
-Returns reference to hash.  Key: iid Value: Remote Device Type
+Returns reference to hash of forwarding table entries status
 
 =back
 
