@@ -36,23 +36,27 @@ use strict;
 use Exporter;
 use SNMP::Info::Layer3;
 use SNMP::Info::MAU;
-use SNMP::Info::LLDP;
 use SNMP::Info::CDP;
+use SNMP::Info::Aggregate 'agg_ports_ifstack';
 
-@SNMP::Info::Layer2::HP::ISA
-    = qw/SNMP::Info::Layer3 SNMP::Info::MAU SNMP::Info::LLDP
-    SNMP::Info::CDP Exporter/;
+@SNMP::Info::Layer2::HP::ISA = qw/
+    SNMP::Info::Aggregate
+    SNMP::Info::Layer3 
+    SNMP::Info::MAU 
+    SNMP::Info::CDP 
+    Exporter
+/;
 @SNMP::Info::Layer2::HP::EXPORT_OK = qw//;
 
 use vars qw/$VERSION %GLOBALS %MIBS %FUNCS %PORTSTAT %MODEL_MAP %MUNGE/;
 
-$VERSION = '2.05';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
     %SNMP::Info::MAU::MIBS,
-    %SNMP::Info::LLDP::MIBS,
     %SNMP::Info::CDP::MIBS,
+    %SNMP::Info::Aggregate::MIBS,
     'RFC1271-MIB'    => 'logDescription',
     'HP-ICF-OID'     => 'hpSwitch4000',
     'STATISTICS-MIB' => 'hpSwitchCpuStat',
@@ -61,19 +65,22 @@ $VERSION = '2.05';
     'HP-ICF-CHASSIS' => 'hpicfSensorObjectId',
     'HP-ICF-BRIDGE'  => 'hpicfBridgeRstpForceVersion',
     'HP-ICF-POE-MIB' => 'hpicfPoePethPsePortCurrent',
+    'SEMI-MIB'       => 'hpHttpMgSerialNumber',
 );
 
 %GLOBALS = (
     %SNMP::Info::Layer3::GLOBALS,
     %SNMP::Info::MAU::GLOBALS,
-    %SNMP::Info::LLDP::GLOBALS,
     %SNMP::Info::CDP::GLOBALS,
+    %SNMP::Info::Aggregate::GLOBALS,
     'serial1'      => 'entPhysicalSerialNum.1',
+    'serial2'      => 'hpHttpMgSerialNumber.0',
     'hp_cpu'       => 'hpSwitchCpuStat.0',
     'hp_mem_total' => 'hpGlobalMemTotalBytes.1',
     'mem_free'     => 'hpGlobalMemFreeBytes.1',
     'mem_used'     => 'hpGlobalMemAllocBytes.1',
     'os_version'   => 'hpSwitchOsVersion.0',
+    'os_version2'  => 'hpHttpMgVersion.0',
     'os_bin'       => 'hpSwitchRomVersion.0',
     'mac'          => 'hpSwitchBaseMACAddress.0',
     'rstp_ver'     => 'hpicfBridgeRstpForceVersion',
@@ -82,8 +89,8 @@ $VERSION = '2.05';
 %FUNCS = (
     %SNMP::Info::Layer3::FUNCS,
     %SNMP::Info::MAU::FUNCS,
-    %SNMP::Info::LLDP::FUNCS,
     %SNMP::Info::CDP::FUNCS,
+    %SNMP::Info::Aggregate::FUNCS,
     'i_type2'   => 'ifType',
 
     # RFC1271
@@ -107,8 +114,8 @@ $VERSION = '2.05';
     # Inherit all the built in munging
     %SNMP::Info::Layer3::MUNGE,
     %SNMP::Info::MAU::MUNGE,
-    %SNMP::Info::LLDP::MUNGE,
     %SNMP::Info::CDP::MUNGE,
+    %SNMP::Info::Aggregate::MUNGE,
     'c_id'   => \&munge_hp_c_id,
 );
 
@@ -118,6 +125,8 @@ $VERSION = '2.05';
 %MODEL_MAP = (
     'J8131A' => 'WAP-420-WW',
     'J8130A' => 'WAP-420-NA',
+    'J9833A' => 'PS1810-8G',
+    'J9834A' => 'PS1810-24G',
     'J8133A' => 'AP520WL',
     'J8680A' => '9408sl',
     'J9091A' => '8212zl',
@@ -132,8 +141,10 @@ $VERSION = '2.05';
     'J8992A' => '6200yl-24G',
     'J4902A' => '6108',
     'J8698A' => '5412zl',
+    'J9851A' => '5412R-zl2',
     'J8719A' => '5408yl',
     'J8697A' => '5406zl',
+    'J9850A' => '5406R-zl2',
     'J8718A' => '5404yl',
     'J4819A' => '5308XL',
     'J4850A' => '5304XL',
@@ -145,6 +156,15 @@ $VERSION = '2.05';
     'J8771A' => '4202vl-48G',
     'J4865A' => '4108GL',
     'J4887A' => '4104GL',
+    'J9588A' => '3800-48G-PoE+-4XG',
+    'J9574A' => '3800-48G-PoE+-4SFP+',
+    'J9586A' => '3800-48G-4XG',
+    'J9576A' => '3800-48G-4SFP+',
+    'J9584A' => '3800-24SFP-2SFP+',
+    'J9587A' => '3800-24G-PoE+-2XG',
+    'J9573A' => '3800-24G-PoE+-2SFP+',
+    'J9585A' => '3800-24G-2XG',
+    'J9575A' => '3800-24G-2SFP+',
     'J8693A' => '3500yl-48G-PWR',
     'J8692A' => '3500yl-24G-PWR',
     'J9473A' => '3500-48-PoE',
@@ -155,6 +175,14 @@ $VERSION = '2.05';
     'J4905A' => '3400cl-24G',
     'J4815A' => '3324XL',
     'J4851A' => '3124',
+    'J9729A' => '2920-48G-PoE+',
+    'J9729A' => '2920-48G-PoE+',
+    'J9728A' => '2920-48G',
+    'J9728A' => '2920-48G',
+    'J9727A' => '2920-24G-PoE+',
+    'J9727A' => '2920-24G-PoE+',
+    'J9726A' => '2920-24G',
+    'J9726A' => '2920-24G',
     'J9562A' => '2915-8G-PoE',
     'J9148A' => '2910al-48G-PoE+',
     'J9147A' => '2910al-48G',
@@ -174,6 +202,11 @@ $VERSION = '2.05';
     'J4900B' => '2626-CR',
     'J4900C' => '2626C',
     'J4900A' => '2626',
+    'J9627A' => '2620-48-PoE+',
+    'J9626A' => '2620-48',
+    'J9624A' => '2620-24-PPoE+',
+    'J9625A' => '2620-24-PoE+',
+    'J9623A' => '2620-24',
     'J9565A' => '2615-8-PoE',
     'J9089A' => '2610-48-PWR',
     'J9088A' => '2610-48',
@@ -181,11 +214,26 @@ $VERSION = '2.05';
     'J9086A' => '2610-24/12PWR',
     'J9085A' => '2610-24',
     'J8762A' => '2600-8-PWR',
+    'J9780A' => '2530-8-PoE+',
+    'J9777A' => '2530-8G',
+    'J9783A' => '2530-8',
+    'J9778A' => '2530-48-PoE+',
+    'J9853A' => '2530-48G-PoE+-2SFP+',
+    'J9772A' => '2530-48G-PoE+',
+    'J9855A' => '2530-48G-2SFP+',
+    'J9775A' => '2530-48G',
+    'J9781A' => '2530-48',
+    'J9779A' => '2530-24-PoE+',
+    'J9854A' => '2530-24G-PoE+-2SFP+',
+    'J9773A' => '2530-24G-PoE+',
+    'J9856A' => '2530-24G-2SFP+',
+    'J9776A' => '2530-24G',
+    'J9782A' => '2530-24',
     'J4813A' => '2524',
-    'J9137A' => '2520-8-PoE',
-    'J9138A' => '2520-24-PoE',
     'J9298A' => '2520G-8-PoE',
     'J9299A' => '2520G-24-PoE',
+    'J9137A' => '2520-8-PoE',
+    'J9138A' => '2520-24-PoE',
     'J4812A' => '2512',
     'J9280A' => '2510G-48',
     'J9279A' => '2510G-24',
@@ -196,6 +244,8 @@ $VERSION = '2.05';
     'J4817A' => '2312',
     'J9449A' => '1810G-8',
     'J9450A' => '1810G-24',
+    'J9802A' => '1810-8G',
+    'J9803A' => '1810-24G',
     'J9029A' => '1800-8G',
     'J9028A' => '1800-24G',
 );
@@ -223,7 +273,7 @@ sub os {
 
 sub os_ver {
     my $hp         = shift;
-    my $os_version = $hp->os_version();
+    my $os_version = $hp->os_version() || $hp->os_version2();
     return $os_version if defined $os_version;
 
     # Some older ones don't have this value,so we cull it from the description
@@ -234,6 +284,16 @@ sub os_ver {
     return;
 }
 
+# Regular managed ProCurve switches have the serial num in entity mib, 
+# the web-managed models in the semi mib (hphttpmanageable).
+sub serial {
+    my $hp = shift;
+
+    my $serial = $hp->serial1() || $hp->serial2() || undef;;
+
+    return $serial;
+}
+
 # Lookup model number, and translate the part number to the common number
 sub model {
     my $hp = shift;
@@ -386,108 +446,6 @@ sub _sensor {
     return $result;
 }
 
-#  Use CDP and/or LLDP
-sub hasCDP {
-    my $hp = shift;
-
-    return $hp->hasLLDP() || $hp->SUPER::hasCDP();
-}
-
-sub c_ip {
-    my $hp      = shift;
-    my $partial = shift;
-
-    my $cdp  = $hp->SUPER::c_ip($partial) || {};
-    my $lldp = $hp->lldp_ip($partial)     || {};
-
-    my %c_ip;
-    foreach my $iid ( keys %$cdp ) {
-        my $ip = $cdp->{$iid};
-        next unless defined $ip;
-
-        $c_ip{$iid} = $ip;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $ip = $lldp->{$iid};
-        next unless defined $ip;
-
-        $c_ip{$iid} = $ip;
-    }
-    return \%c_ip;
-}
-
-sub c_if {
-    my $hp      = shift;
-    my $partial = shift;
-
-    my $lldp = $hp->lldp_if($partial)     || {};
-    my $cdp  = $hp->SUPER::c_if($partial) || {};
-
-    my %c_if;
-    foreach my $iid ( keys %$cdp ) {
-        my $if = $cdp->{$iid};
-        next unless defined $if;
-
-        $c_if{$iid} = $if;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $if = $lldp->{$iid};
-        next unless defined $if;
-
-        $c_if{$iid} = $if;
-    }
-    return \%c_if;
-}
-
-sub c_port {
-    my $hp      = shift;
-    my $partial = shift;
-
-    my $lldp = $hp->lldp_port($partial)     || {};
-    my $cdp  = $hp->SUPER::c_port($partial) || {};
-
-    my %c_port;
-    foreach my $iid ( keys %$cdp ) {
-        my $port = $cdp->{$iid};
-        next unless defined $port;
-
-        $c_port{$iid} = $port;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $port = $lldp->{$iid};
-        next unless defined $port;
-        $c_port{$iid} = $port;
-    }
-    return \%c_port;
-}
-
-sub c_id {
-    my $hp      = shift;
-    my $partial = shift;
-
-    my $lldp = $hp->lldp_id($partial)     || {};
-    my $cdp  = $hp->SUPER::c_id($partial) || {};
-
-    my %c_id;
-    foreach my $iid ( keys %$cdp ) {
-        my $id = $cdp->{$iid};
-        next unless defined $id;
-	
-        $c_id{$iid} = $id;
-    }
-    
-    foreach my $iid ( keys %$lldp ) {
-	my $id = $lldp->{$iid};
-	next unless defined $id;
-	
-	$c_id{$iid} = $id;
-    }
-    return \%c_id;
-}
-
 sub munge_hp_c_id {
     my ($v) = @_;
     if ( length(unpack('H*', $v)) == 12 ){
@@ -501,30 +459,6 @@ sub munge_hp_c_id {
     }
 }
 
-sub c_platform {
-    my $hp      = shift;
-    my $partial = shift;
-
-    my $lldp = $hp->lldp_rem_sysdesc($partial)  || {};
-    my $cdp  = $hp->SUPER::c_platform($partial) || {};
-
-    my %c_platform;
-    foreach my $iid ( keys %$cdp ) {
-        my $platform = $cdp->{$iid};
-        next unless defined $platform;
-
-        $c_platform{$iid} = $platform;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $platform = $lldp->{$iid};
-        next unless defined $platform;
-
-        $c_platform{$iid} = $platform;
-    }
-    return \%c_platform;
-}
-
 # POWER-ETHERNET-MIB doesn't define a mapping of its
 # "module"/"port" index to ifIndex.  Different vendors
 # do this in different ways.
@@ -612,8 +546,11 @@ sub set_i_vlan_tagged {
             $hp->error_throw(sprintf("Requested VLAN %s doesn't seem to exist on device...", $vlan));
         }
     }
+    return;
 }
 
+sub agg_ports { return agg_ports_ifstack(@_) }
+
 1;
 __END__
 
@@ -660,8 +597,6 @@ after determining a more specific class using the method above.
 
 =item SNMP::Info::Layer2
 
-=item SNMP::Info::LLDP
-
 =item SNMP::Info::MAU
 
 =back
@@ -731,7 +666,7 @@ Returns bytes of used memory
 Returns the model number of the HP Switch.  Will translate between the HP Part
 number and the common model number with this map :
 
-%MODEL_MAP = (
+ %MODEL_MAP = (
     'J8131A' => 'WAP-420-WW',
     'J8130A' => 'WAP-420-NA',
     'J8133A' => 'AP520WL',
@@ -761,6 +696,15 @@ number and the common model number with this map :
     'J8771A' => '4202vl-48G',
     'J4865A' => '4108GL',
     'J4887A' => '4104GL',
+    'J9588A' => '3800-48G-PoE+-4XG',
+    'J9574A' => '3800-48G-PoE+-4SFP+',
+    'J9586A' => '3800-48G-4XG',
+    'J9576A' => '3800-48G-4SFP+',
+    'J9584A' => '3800-24SFP-2SFP+',
+    'J9587A' => '3800-24G-PoE+-2XG',
+    'J9573A' => '3800-24G-PoE+-2SFP+',
+    'J9585A' => '3800-24G-2XG',
+    'J9575A' => '3800-24G-2SFP+',
     'J8693A' => '3500yl-48G-PWR',
     'J8692A' => '3500yl-24G-PWR',
     'J9473A' => '3500-48-PoE',
@@ -771,6 +715,7 @@ number and the common model number with this map :
     'J4905A' => '3400cl-24G',
     'J4815A' => '3324XL',
     'J4851A' => '3124',
+    'J9562A' => '2915-8G-PoE',
     'J9148A' => '2910al-48G-PoE+',
     'J9147A' => '2910al-48G',
     'J9146A' => '2910al-24G-PoE+',
@@ -789,6 +734,12 @@ number and the common model number with this map :
     'J4900B' => '2626-CR',
     'J4900C' => '2626C',
     'J4900A' => '2626',
+    'J9627A' => '2620-48-PoE+',
+    'J9626A' => '2620-48',
+    'J9624A' => '2620-24-PPoE+',
+    'J9625A' => '2620-24-PoE+',
+    'J9623A' => '2620-24',
+    'J9565A' => '2615-8-PoE',
     'J9089A' => '2610-48-PWR',
     'J9088A' => '2610-48',
     'J9087A' => '2610-24-PWR',
@@ -796,6 +747,8 @@ number and the common model number with this map :
     'J9085A' => '2610-24',
     'J8762A' => '2600-8-PWR',
     'J4813A' => '2524',
+    'J9298A' => '2520G-8-PoE',
+    'J9299A' => '2520G-24-PoE',
     'J9137A' => '2520-8-PoE',
     'J9138A' => '2520-24-PoE',
     'J4812A' => '2512',
@@ -806,9 +759,11 @@ number and the common model number with this map :
     'J9019A' => '2510-24A',
     'J4818A' => '2324',
     'J4817A' => '2312',
+    'J9449A' => '1810G-8',
+    'J9450A' => '1810G-24',
     'J9029A' => '1800-8G',
     'J9028A' => '1800-24G',
-);
+ );
 
 =item $hp->os()
 
@@ -827,6 +782,10 @@ the description field.
 
 C<hpSwitchOsVersion.0>
 
+=item $hp->serial()
+
+Returns serial number if available through SNMP
+
 =item $hp->slots()
 
 Returns number of entries in $hp->e_name that have 'slot' in them.
@@ -850,12 +809,12 @@ Power supply 2 status
 =item $hp->peth_port_power()
 
 Power supplied by PoE ports, in milliwatts
-("hpicfPoePethPsePortPower")
+(C<hpicfPoePethPsePortPower>)
 
 =item $hp->stp_ver()
 
 Returns what version of STP the device is running.
-("hpicfBridgeRstpForceVersion" with fallback to inherited stp_ver())
+(C<hpicfBridgeRstpForceVersion> with fallback to inherited stp_ver())
 
 =back
 
@@ -863,10 +822,6 @@ Returns what version of STP the device is running.
 
 See documentation in L<SNMP::Info::Layer2/"GLOBALS"> for details.
 
-=head2 Globals imported from SNMP::Info::LLDP
-
-See documentation in L<SNMP::Info::LLDP/"GLOBALS"> for details.
-
 =head2 Globals imported from SNMP::Info::MAU
 
 See documentation in L<SNMP::Info::MAU/"GLOBALS"> for details.
@@ -878,7 +833,7 @@ to a hash.
 
 =head2 Overrides
 
-=over
+=over 4
 
 =item $hp->interfaces() 
 
@@ -894,7 +849,8 @@ Returns reference to hash of IIDs to admin duplex setting.
 
 =item $hp->vendor_i_type()
 
-Returns reference to hash of IIDs to HP specific port type (hpSwitchPortType).
+Returns reference to hash of IIDs to HP specific port type
+(C<hpSwitchPortType>).
 
 =item $hp->i_name()
 
@@ -905,52 +861,11 @@ Crosses i_name() with $hp->e_name() using $hp->e_port() and i_alias()
 Returns reference to hash of power Ethernet port table entries map back to
 interface index (c<ifIndex>)
 
-=back
+=item C<agg_ports>
 
-=head2 Topology information
-
-Based upon the firmware version HP devices may support Cisco Discovery
-Protocol (CDP), Link Layer Discovery Protocol (LLDP), or both.  These methods
-will query both and return the combination of all information.  As a result,
-there may be identical topology information returned from the two protocols
-causing duplicate entries.  It is the calling program's responsibility to
-identify any duplicate entries and remove duplicates if necessary.
-
-=over
-
-=item $hp->hasCDP()
-
-Returns true if the device is running either CDP or LLDP.
-
-=item $hp->c_if()
-
-Returns reference to hash.  Key: iid Value: local device port (interfaces)
-
-=item $hp->c_ip()
-
-Returns reference to hash.  Key: iid Value: remote IPv4 address
-
-If multiple entries exist with the same local port, c_if(), with the same IPv4
-address, c_ip(), it may be a duplicate entry.
-
-If multiple entries exist with the same local port, c_if(), with different
-IPv4 addresses, c_ip(), there is either a non-CDP/LLDP device in between two
-or more devices or multiple devices which are not directly connected.  
-
-Use the data from the Layer2 Topology Table below to dig deeper.
-
-=item $hp->c_port()
-
-Returns reference to hash. Key: iid Value: remote port (interfaces)
-
-=item $hp->c_id()
-
-Returns reference to hash. Key: iid Value: string value used to identify the
-chassis component associated with the remote system.
-
-=item $hp->c_platform()
-
-Returns reference to hash.  Key: iid Value: Remote Device Type
+Returns a HASH reference mapping from slave to master port for each member of
+a port bundle on the device. Keys are ifIndex of the slave ports, Values are
+ifIndex of the corresponding master ports.
 
 =back
 
@@ -958,10 +873,6 @@ Returns reference to hash.  Key: iid Value: Remote Device Type
 
 See documentation in L<SNMP::Info::Layer2/"TABLE METHODS"> for details.
 
-=head2 Table Methods imported from SNMP::Info::LLDP
-
-See documentation in L<SNMP::Info::LLDP/"TABLE METHODS"> for details.
-
 =head2 Table Methods imported from SNMP::Info::MAU
 
 See documentation in L<SNMP::Info::MAU/"TABLE METHODS"> for details.
@@ -989,4 +900,6 @@ operations.
 
 =item set_i_vlan_tagged()
 
+=back
+
 =cut
