@@ -36,19 +36,25 @@ use Exporter;
 use SNMP::Info::Layer3;
 use SNMP::Info::MAU;
 use SNMP::Info::LLDP;
+use SNMP::Info::Aggregate 'agg_ports_ifstack';
 
-@SNMP::Info::Layer3::Arista::ISA = qw/SNMP::Info::LLDP SNMP::Info::MAU
-    SNMP::Info::Layer3 Exporter/;
+@SNMP::Info::Layer3::Arista::ISA = qw/
+    SNMP::Info::Aggregate
+    SNMP::Info::LLDP
+    SNMP::Info::MAU
+    SNMP::Info::Layer3 Exporter
+/;
 @SNMP::Info::Layer3::Arista::EXPORT_OK = qw//;
 
 use vars qw/$VERSION %GLOBALS %MIBS %FUNCS %MUNGE/;
 
-$VERSION = '2.05';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
     %SNMP::Info::MAU::MIBS,
     %SNMP::Info::LLDP::MIBS,
+    %SNMP::Info::Aggregate::MIBS,
     'ARISTA-PRODUCTS-MIB' => 'aristaProducts',
 );
 
@@ -104,64 +110,28 @@ sub model {
     return $model;
 }
 
-# Use Q-BRIDGE-MIB
-
-sub fw_mac {
+# The LLDP MIB leaves it up in the air what the index means.
+# On EOS, it's a dot1d port.
+sub lldp_if {
     my $arista  = shift;
     my $partial = shift;
 
-    return $arista->qb_fw_mac($partial);
+    # We pick a column that someone else is likely to want,
+    # so that the cache means that hopefully this doesn't
+    # cause any more SNMP transactions in total.
+    my $desc     = $arista->lldp_rem_desc($partial) || {};
+    my $bp_index = $arista->bp_index() || {};
+
+    my $lldp_if = {};
+    foreach my $key ( keys %$desc ) {
+        my @aOID = split( '\.', $key );
+        my $port = $aOID[1];
+        $lldp_if->{$key} = $bp_index->{$port};
+    }
+    return $lldp_if;
 }
 
-sub fw_port {
-    my $arista  = shift;
-    my $partial = shift;
-
-    return $arista->qb_fw_port($partial);
-}
-
-# Use LLDP
-
-sub hasCDP {
-    my $arista = shift;
-
-    return $arista->hasLLDP();
-}
-
-sub c_ip {
-    my $arista  = shift;
-    my $partial = shift;
-
-    return $arista->lldp_ip($partial);
-}
-
-sub c_if {
-    my $arista  = shift;
-    my $partial = shift;
-
-    return $arista->lldp_if($partial);
-}
-
-sub c_port {
-    my $arista  = shift;
-    my $partial = shift;
-
-    return $arista->lldp_port($partial);
-}
-
-sub c_id {
-    my $arista  = shift;
-    my $partial = shift;
-
-    return $arista->lldp_id($partial);
-}
-
-sub c_platform {
-    my $arista  = shift;
-    my $partial = shift;
-
-    return $arista->lldp_rem_sysdesc($partial);
-}
+sub agg_ports { return agg_ports_ifstack(@_) }
 
 1;
 __END__
@@ -198,6 +168,8 @@ Subclass for Arista Networks EOS-based devices
 
 =over
 
+=item SNMP::Info::Aggregate
+
 =item SNMP::Info::Layer3
 
 =item SNMP::Info::MAU
@@ -216,6 +188,8 @@ Subclass for Arista Networks EOS-based devices
 
 See L<SNMP::Info::Layer3/"Required MIBs"> for its own MIB requirements.
 
+See L<SNMP::Info::Aggregate/"Required MIBs"> for its own MIB requirements.
+
 See L<SNMP::Info::MAU/"Required MIBs"> for its own MIB requirements.
 
 See L<SNMP::Info::LLDP/"Required MIBs"> for its own MIB requirements.
@@ -232,10 +206,6 @@ These are methods that return scalar values from SNMP
 
     Returns 'Arista Networks, Inc.'
 
-=item $arista->hasCDP()
-
-    Returns whether LLDP is enabled.
-
 =item $arista->model()
 
 Tries to reference $arista->id() to one of the product MIBs listed above
@@ -271,34 +241,6 @@ to a hash.
 
 =over
 
-=item $arista->fw_mac()
-
-Use the F<Q-BRIDGE-MIB> instead of F<BRIDGE-MIB>
-
-=item $arista->fw_port()
-
-Use the F<Q-BRIDGE-MIB> instead of F<BRIDGE-MIB>
-
-=item $arista->c_id()
-
-Returns LLDP information.
-
-=item $arista->c_if()
-
-Returns LLDP information.
-
-=item $arista->c_ip()
-
-Returns LLDP information.
-
-=item $arista->c_platform()
-
-Returns LLDP information.
-
-=item $arista->c_port()
-
-Returns LLDP information.
-
 =item $arista->i_duplex_admin()
 
 Returns info from F<MAU-MIB>
@@ -307,6 +249,16 @@ Returns info from F<MAU-MIB>
 
 Returns info from F<MAU-MIB>
 
+=item $arista->lldp_if()
+
+Returns the mapping to the SNMP Interface Table.
+
+=item C<agg_ports>
+
+Returns a HASH reference mapping from slave to master port for each member of
+a port bundle on the device. Keys are ifIndex of the slave ports, Values are
+ifIndex of the corresponding master ports.
+
 =back
 
 =head2 Table Methods imported from SNMP::Info::Layer3
