@@ -43,7 +43,7 @@ use SNMP::Info::Bridge;
 use vars qw/$VERSION %GLOBALS %FUNCS %MIBS %MUNGE %MODEL_MAP
     %MODID_MAP %PROCID_MAP/;
 
-$VERSION = '2.05';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::MIBS,
@@ -545,7 +545,7 @@ sub model {
 }
 
 sub vendor {
-    return 'nortel';
+    return 'avaya';
 }
 
 sub os {
@@ -732,6 +732,8 @@ sub i_vlan {
     return \%i_vlan;
 }
 
+sub i_pvid { goto &i_vlan }
+
 sub root_ip {
     my $bayrs = shift;
 
@@ -1191,7 +1193,7 @@ sub e_vendor {
 
     my %wf_e_vendor;
     foreach my $iid ( keys %$wf_e_idx ) {
-        $wf_e_vendor{$iid} = 'nortel';
+        $wf_e_vendor{$iid} = 'avaya';
     }
     return \%wf_e_vendor;
 }
@@ -1378,7 +1380,8 @@ __END__
 
 =head1 NAME
 
-SNMP::Info::Layer3::BayRS - SNMP Interface to Nortel routers running BayRS.
+SNMP::Info::Layer3::BayRS - SNMP Interface to Avaya/Nortel routers running
+BayRS.
 
 =head1 AUTHOR
 
@@ -1401,7 +1404,7 @@ Eric Miller
 
 =head1 DESCRIPTION
 
-Abstraction subclass for routers running Nortel BayRS.  
+Abstraction subclass for routers running Avaya/Nortel BayRS.  
 
 For speed or debugging purposes you can call the subclass directly, but not
 after determining a more specific class using the method above. 
@@ -1474,7 +1477,7 @@ and the common model with this map :
 
 =item $bayrs->vendor()
 
-Returns 'nortel'
+Returns 'avaya'
 
 =item $bayrs->os()
 
@@ -1548,6 +1551,10 @@ interfaces.
 
 Returns reference to hash.  Maps port VLAN ID to IIDs.
 
+=item $bayrs->i_pvid()
+
+Returns reference to hash.  Maps port VLAN ID to IIDs.
+
 =back
 
 =head2 Pseudo F<ENTITY-MIB> information
@@ -1583,7 +1590,7 @@ Returns reference to hash.  Key: IID, Value: Hardware version.
 
 =item $bayrs->e_vendor()
 
-Returns reference to hash.  Key: IID, Value: nortel.
+Returns reference to hash.  Key: IID, Value: avaya.
 
 =item $bayrs->e_serial()
 
@@ -1634,11 +1641,11 @@ See documentation in L<SNMP::Info::Layer3/"TABLE METHODS"> for details.
 
 =over
 
-=item $extreme->munge_hw_rev()
+=item $bayrs->munge_hw_rev()
 
 Converts octets to a decimal major.minor string.
 
-=item $extreme->munge_wf_serial()
+=item $bayrs->munge_wf_serial()
 
 Coverts octets to a decimal string.
 
