@@ -33,16 +33,18 @@ package SNMP::Info::Layer3::Dell;
 use strict;
 use Exporter;
 use SNMP::Info::Layer3;
+use SNMP::Info::LLDP;
 
-@SNMP::Info::Layer3::Dell::ISA       = qw/SNMP::Info::Layer3 Exporter/;
+@SNMP::Info::Layer3::Dell::ISA       = qw/SNMP::Info::LLDP SNMP::Info::Layer3 Exporter/;
 @SNMP::Info::Layer3::Dell::EXPORT_OK = qw//;
 
 use vars qw/$VERSION %GLOBALS %FUNCS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
+    %SNMP::Info::LLDP::MIBS,
     'RADLAN-Physicaldescription-MIB' => 'rlPhdStackReorder',
     'RADLAN-rlInterfaces'            => 'rlIfNumOfLoopbackPorts',
     'RADLAN-HWENVIROMENT'            => 'rlEnvPhysicalDescription',
@@ -51,12 +53,14 @@ $VERSION = '2.06';
 
 %GLOBALS = (
     %SNMP::Info::Layer3::GLOBALS,
+    %SNMP::Info::LLDP::GLOBALS,
     'os_ver'       => 'productIdentificationVersion',
     'dell_id_name' => 'productIdentificationDisplayName',
 );
 
 %FUNCS = (
     %SNMP::Info::Layer3::FUNCS,
+    %SNMP::Info::LLDP::FUNCS,
 
     # RADLAN-rlInterfaces:swIfTable
     'dell_duplex_admin' => 'swIfDuplexAdminMode',
@@ -102,7 +106,7 @@ $VERSION = '2.06';
     'dell_fan_desc'  => 'rlEnvMonFanStatusDescr',
 );
 
-%MUNGE = ( %SNMP::Info::Layer3::MUNGE, );
+%MUNGE = ( %SNMP::Info::Layer3::MUNGE, %SNMP::Info::LLDP::MUNGE, );
 
 # Method OverRides
 
@@ -149,6 +153,48 @@ sub serial {
     return $dell->SUPER::serial();
 }
 
+# check all fans, and report overall status
+sub fan {
+    my $dell = shift;
+
+    my $fan   = $dell->dell_fan_desc()  || {};
+    my $state = $dell->dell_fan_state() || {};
+    my @messages = ();
+
+    foreach my $k (keys %$fan) {
+        next if $state->{$k} and $state->{$k} eq 'normal';
+        push @messages, "$fan->{$k}: $state->{$k}";
+    }
+
+    push @messages, ((scalar keys %$fan). " fans OK")
+      if scalar @messages == 0;
+
+    return (join ", ", @messages);
+}
+
+sub _ps_status {
+    my ($dell, $unit) = @_;
+
+    my $status = 'unknown';
+    return $status if !defined $unit;
+
+    my $desc  = $dell->dell_pwr_desc()  || {};
+    my $state = $dell->dell_pwr_state() || {};
+
+    foreach my $k (keys %$desc) {
+        next unless $desc->{$k} and $desc->{$k} eq "ps1_unit$unit";
+        return ($state->{$k} || $status);
+    }
+
+    return $status;
+}
+
+sub ps1_type { return 'internalRedundant' }
+sub ps2_type { return 'internalRedundant' }
+
+sub ps1_status { return (shift)->_ps_status(1) }
+sub ps2_status { return (shift)->_ps_status(2) }
+
 sub interfaces {
     my $dell    = shift;
     my $partial = shift;
@@ -189,25 +235,6 @@ sub i_duplex_admin {
     return \%i_duplex_admin;
 }
 
-# Use same methods as netgear.  Some device didn't implement the bridge MIB
-# forwarding table and some don't return MACs for VLANs other than default yet
-# don't support community indexing, so we use the Q-BRIDGE-MIB forwarding
-# table.  Fall back to the orig functions if the qb versions don't
-# return anything.
-sub fw_mac {
-    my $dell = shift;
-    my $ret  = $dell->qb_fw_mac();
-    $ret = $dell->orig_fw_mac() if ( !defined($ret) );
-    return $ret;
-}
-
-sub fw_port {
-    my $dell = shift;
-    my $ret  = $dell->qb_fw_port();
-    $ret = $dell->orig_fw_port() if ( !defined($ret) );
-    return $ret;
-}
-
 sub _vendor {
     my $dell = shift;
 
@@ -228,6 +255,10 @@ sub _vendor {
     }
 }
 
+# dot1qTpFdbTable uses dot1qVlanIndex rather than dot1qFdbId as index,
+# so pretend we don't have the mapping
+sub qb_fdb_index {return}
+
 1;
 __END__
 
@@ -323,6 +354,26 @@ id().  Defaults to 'dlink'.
 Returns 'dell', 'dlink', or 'ibm' based upon the IANA enterprise number in
 id().  Defaults to 'dlink'.
 
+=item $dell->fan()
+
+Return the status of all fans from the F<Dell-Vendor-MIB>
+
+=item $dell->ps1_type()
+
+Return the type of the first power supply from the F<Dell-Vendor-MIB>
+
+=item $dell->ps2_type()
+
+Return the type of the second power supply from the F<Dell-Vendor-MIB>
+
+=item $dell->ps1_status()
+
+Return the status of the first power supply from the F<Dell-Vendor-MIB>
+
+=item $dell->ps2_status()
+
+Return the status of the second power supply from the F<Dell-Vendor-MIB>
+
 =back
 
 =head2 Overrides
@@ -340,6 +391,10 @@ otherwise uses the Layer3 serial method.
 
 See documentation in L<SNMP::Info::Layer3/"GLOBALS"> for details.
 
+=head2 Globals imported from SNMP::Info::LLDP
+
+See documentation in L<SNMP::Info::LLDP/"GLOBALS"> for details.
+
 =head1 TABLE METHODS
 
 These are methods that return tables of information in the form of a reference
@@ -398,22 +453,9 @@ sometimes not unique.
 Returns reference to hash of iid to current link administrative duplex
 setting.
 
-=item $dell->fw_mac()
+=item $dell->qb_fdb_index()
 
-Returns reference to hash of forwarding table MAC Addresses.
-
-Some devices don't implement the C<BRIDGE-MIB> forwarding table, so we use
-the C<Q-BRIDGE-MIB> forwarding table.  Fall back to the C<BRIDGE-MIB> if
-C<Q-BRIDGE-MIB> doesn't return anything.
-
-=item $dell->fw_port()
-
-Returns reference to hash of forwarding table entries port interface
-identifier (iid)
-
-Some devices don't implement the C<BRIDGE-MIB> forwarding table, so we use
-the C<Q-BRIDGE-MIB> forwarding table.  Fall back to the C<BRIDGE-MIB> if
-C<Q-BRIDGE-MIB> doesn't return anything.
+Returns nothing to work around incorrect indexing of C<dot1qTpFdbTable>
 
 =back
 
@@ -421,4 +463,8 @@ C<Q-BRIDGE-MIB> doesn't return anything.
 
 See documentation in L<SNMP::Info::Layer3/"TABLE METHODS"> for details.
 
+=head2 Table Methods imported from SNMP::Info::LLDP
+
+See documentation in L<SNMP::Info::LLDP/"TABLE METHODS"> for details.
+
 =cut
