@@ -44,7 +44,7 @@ use SNMP::Info::Layer3;
 
 use vars qw/$VERSION $DEBUG %GLOBALS %FUNCS $INIT %MIBS %MUNGE/;
 
-$VERSION = '2.05';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS, %SNMP::Info::CDP::MIBS,
@@ -149,154 +149,103 @@ sub i_duplex_admin {
     return $enterasys->mau_i_duplex_admin($partial);
 }
 
-# Normal BRIDGE-MIB has issues on some devices, duplicates and
-# non-increasing oids, Use Q-BRIDGE-MIB for macsuck
-sub fw_mac {
-    my $enterasys = shift;
-    my $partial   = shift;
-
-    return $enterasys->qb_fw_mac($partial);
-}
-
-sub fw_port {
-    my $enterasys = shift;
-    my $partial   = shift;
-
-    return $enterasys->qb_fw_port($partial);
-}
-
-#  Use CDP and/or LLDP
-#
-#  LLDP table timefilter implementation continuously increments when walked
+#  TimeFilter implementation continuously increments when walked
 #  and we may never reach the end of the table.  This behavior can be
 #  modified with the "set snmp timefilter break disable" command,
 #  unfortunately it is not the default.  Query with a partial value of zero
-#  which means no time filter.
+#  which means no time filter for tables with and index containing a
+#  TimeFilter
 
-sub hasCDP {
-    my $enterasys = shift;
+sub qb_fdb_index {
+    my $bridge  = shift;
 
-    return $enterasys->hasLLDP() || $enterasys->SUPER::hasCDP();
+    my $qb_fdb_ids = $bridge->dot1qVlanFdbId(0) || {};
+
+    # Strip the TimeFilter
+    my $vl_fdb_index = {};
+    for my $fdb_entry (keys(%$qb_fdb_ids)) {
+        (my $vlan = $fdb_entry) =~ s/^\d+\.//;
+        $vl_fdb_index->{$qb_fdb_ids->{$fdb_entry}} = $vlan;
+    }
+
+    return $vl_fdb_index;
 }
 
-sub c_ip {
-    my $enterasys = shift;
-    my $partial   = shift;
+sub i_vlan_membership {
+    my $bridge  = shift;
+    my $partial = shift;
 
-    my $cdp  = $enterasys->SUPER::c_ip($partial) || {};
-    my $lldp = $enterasys->lldp_ip(0)            || {};
+    # dot1qVlanCurrentTable TimeFilter index
+    my $v_ports = $bridge->qb_cv_egress(0) || $bridge->qb_v_egress();
 
-    my %c_ip;
-    foreach my $iid ( keys %$cdp ) {
-        my $ip = $cdp->{$iid};
-        next unless defined $ip;
-
-        $c_ip{$iid} = $ip;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $ip = $lldp->{$iid};
-        next unless defined $ip;
-
-        $c_ip{$iid} = $ip;
-    }
-    return \%c_ip;
+    return $bridge->_vlan_hoa($v_ports, $partial);
 }
 
-sub c_if {
-    my $enterasys = shift;
-    my $partial   = shift;
+sub i_vlan_membership_untagged {
+    my $bridge  = shift;
+    my $partial = shift;
 
-    my $lldp = $enterasys->lldp_if(0)            || {};
-    my $cdp  = $enterasys->SUPER::c_if($partial) || {};
+    # dot1qVlanCurrentTable TimeFilter index
+    my $v_ports = $bridge->qb_cv_untagged(0) || $bridge->qb_v_untagged();
 
-    my %c_if;
-    foreach my $iid ( keys %$cdp ) {
-        my $if = $cdp->{$iid};
-        next unless defined $if;
-
-        $c_if{$iid} = $if;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $if = $lldp->{$iid};
-        next unless defined $if;
-
-        $c_if{$iid} = $if;
-    }
-    return \%c_if;
+    return $bridge->_vlan_hoa($v_ports, $partial);
 }
 
-sub c_port {
+sub lldp_ip {
     my $enterasys = shift;
-    my $partial   = shift;
+    my $partial   = shift || 0;
 
-    my $lldp = $enterasys->lldp_port(0)            || {};
-    my $cdp  = $enterasys->SUPER::c_port($partial) || {};
-
-    my %c_port;
-    foreach my $iid ( keys %$cdp ) {
-        my $port = $cdp->{$iid};
-        next unless defined $port;
-
-        $c_port{$iid} = $port;
-    }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $port = $lldp->{$iid};
-        next unless defined $port;
-
-        $c_port{$iid} = $port;
-    }
-    return \%c_port;
+    return $enterasys->SUPER::lldp_ip($partial);
 }
 
-sub c_id {
-    my $enterasys = shift;
-    my $partial   = shift;
+# [3564920] LLDP-MIB::lldpLocPortDesc isn't always unique,
+# use LLDP-MIB::lldpLocPortId this cross references to ifName
+sub lldp_if {
+    my $lldp    = shift;
+    my $partial = shift || 0;
 
-    my $lldp = $enterasys->lldp_id(0)            || {};
-    my $cdp  = $enterasys->SUPER::c_id($partial) || {};
-
-    my %c_id;
-    foreach my $iid ( keys %$cdp ) {
-        my $id = $cdp->{$iid};
-        next unless defined $id;
-
-        $c_id{$iid} = $id;
+    my $addr    = $lldp->lldp_rem_pid($partial) || {};
+    my $i_descr = $lldp->ifName() || {};
+    my %r_i_descr = reverse %$i_descr;
+    
+    my %lldp_if;
+    foreach my $key ( keys %$addr ) {
+        my @aOID = split( '\.', $key );
+        my $port = $aOID[1];
+        next unless $port;
+        # Local LLDP port may not equate to ifIndex
+        # Cross reference lldpLocPortId with ifName to get ifIndex
+        my $lldp_desc = $lldp->lldpLocPortId($port);
+        my $desc = $lldp_desc->{$port};
+        # If cross reference is successful use it, otherwise stick with lldpRemLocalPortNum
+        if ( exists $r_i_descr{$desc} ) {
+            $port = $r_i_descr{$desc};
+        }
+        
+        $lldp_if{$key} = $port;
     }
-
-    foreach my $iid ( keys %$lldp ) {
-        my $id = $lldp->{$iid};
-        next unless defined $id;
-
-        $c_id{$iid} = $id;
-    }
-    return \%c_id;
+    return \%lldp_if;
 }
 
-sub c_platform {
+sub lldp_port {
     my $enterasys = shift;
-    my $partial   = shift;
+    my $partial   = shift || 0;
 
-    my $lldp = $enterasys->lldp_rem_sysdesc(0)         || {};
-    my $cdp  = $enterasys->SUPER::c_platform($partial) || {};
+    return $enterasys->SUPER::lldp_port($partial);
+}
 
-    my %c_platform;
-    foreach my $iid ( keys %$cdp ) {
-        my $platform = $cdp->{$iid};
-        next unless defined $platform;
+sub lldp_id {
+    my $enterasys = shift;
+    my $partial   = shift || 0;
 
-        $c_platform{$iid} = $platform;
-    }
+    return  $enterasys->SUPER::lldp_id($partial);
+}
 
-    foreach my $iid ( keys %$lldp ) {
-        my $platform = $lldp->{$iid};
-        next unless defined $platform;
+sub lldp_platform {
+    my $enterasys = shift;
+    my $partial   = shift || 0;
 
-        $c_platform{$iid} = $platform;
-    }
-    return \%c_platform;
+    return  $enterasys->SUPER::lldp_rem_sysdesc($partial);
 }
 
 1;
@@ -399,12 +348,6 @@ Returns base mac
 
 =back
 
-=head2 Overrides
-
-=over
-
-=back
-
 =head2 Globals imported from SNMP::Info::MAU
 
 See documentation in L<SNMP::Info::MAU/"GLOBALS"> for details.
@@ -450,65 +393,33 @@ See documentation for mau_i_duplex() in L<SNMP::Info::MAU/"TABLE METHODS">.
 See documentation for mau_i_duplex_admin() in
 L<SNMP::Info::MAU/"TABLE METHODS">.
 
-=item $enterasys->fw_mac()
-
-Returns reference to hash of forwarding table MAC Addresses.
-
-=item $enterasys->fw_port()
-
-Returns reference to hash of forwarding table entries port interface
-identifier (iid).
-
-(C<dot1qTpFdbPort>)
-
 =back
 
-=head2 Topology information
+=head2 Time Filter Table Index Overrides
 
-Based upon the firmware version Enterasys devices may support Cabletron
-Discovery Protocol (CTRON CDP), Cisco Discovery Protocol (CDP), Link Layer
-Discovery Protocol (LLDP), or all.  This module currently supports CDP and
-LLDP, but not CTRON CDP.  These methods will query both CDP and LLDP and
-return the combination of all information.  As a result, there may be
-identical topology information returned from the two protocols
-causing duplicate entries.  It is the calling program's responsibility to
-identify any duplicate entries and remove duplicates if necessary.
+The time filter C<TimeFilter> implementation continuously increments when
+walked and we may never reach the end of the table.  This behavior can be
+modified with the C<"set snmp timefilter break disable"> command,
+unfortunately it is not the default.  These methods are overridden to
+supply a partial value of zero which means no time filter.
 
 =over
 
-=item $enterasys->hasCDP()
+=item $enterasys->qb_fdb_index()
 
-Returns true if the device is running either CDP or LLDP.
+=item $enterasys->i_vlan_membership()
 
-=item $enterasys->c_if()
+=item $enterasys->i_vlan_membership_untagged()
 
-Returns reference to hash.  Key: iid Value: local device port (interfaces)
+=item $enterasys->lldp_if()
 
-=item $enterasys->c_ip()
+=item $enterasys->lldp_ip()
 
-Returns reference to hash.  Key: iid Value: remote IPv4 address
+=item $enterasys->lldp_port()
 
-If multiple entries exist with the same local port, c_if(), with the same IPv4
-address, c_ip(), it may be a duplicate entry.
+=item $enterasys->lldp_id()
 
-If multiple entries exist with the same local port, c_if(), with different
-IPv4 addresses, c_ip(), there is either a non-CDP/LLDP device in between two
-or more devices or multiple devices which are not directly connected.  
-
-Use the data from the Layer2 Topology Table below to dig deeper.
-
-=item $enterasys->c_port()
-
-Returns reference to hash. Key: iid Value: remote port (interfaces)
-
-=item $enterasys->c_id()
-
-Returns reference to hash. Key: iid Value: string value used to identify the
-chassis component associated with the remote system.
-
-=item $enterasys->c_platform()
-
-Returns reference to hash.  Key: iid Value: Remote Device Type
+=item $enterasys->lldp_platform()
 
 =back
 
