@@ -1,6 +1,6 @@
 # SNMP::Info::Layer3::Passport
 #
-# Copyright (c) 2012 Eric Miller
+# Copyright (c) 2016 Eric Miller
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,7 @@ use SNMP::Info::Layer3;
 
 use vars qw/$VERSION %GLOBALS %FUNCS %MIBS %MUNGE/;
 
-$VERSION = '3.28';
+$VERSION = '3.34';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS, %SNMP::Info::RapidCity::MIBS,
@@ -174,9 +174,11 @@ sub interfaces {
     my $partial  = shift;
 
     my $i_index      = $passport->i_index($partial);
+    my $i_descr      = $passport->orig_i_description($partial) || {};
     my $model        = $passport->model();
     my $index_factor = $passport->index_factor();
     my $port_offset  = $passport->port_offset();
+    my $slot_offset  = $passport->slot_offset();
     my $vlan_index   = {};
     my %reverse_vlan;
     my $vlan_id = {};
@@ -202,6 +204,10 @@ sub interfaces {
             $if{$index} = 'Cpu.Virtual';
         }
 
+        elsif ( ( $iid == 64 ) and ( $model =~ /^VSP[478]/ ) ) {
+            $if{$index} = 'Mgmt.1';
+        }
+
         elsif ( ( $index == 192 ) and ( $model =~ /^8[86]03/ ) ) {
             $if{$index} = 'Cpu.3';
         }
@@ -226,11 +232,18 @@ sub interfaces {
         }
 
         else {
-            my $port = ( $index % $index_factor ) + $port_offset;
-            my $slot = int( $index / $index_factor );
+            if ($model =~ /VSP/ and $i_descr->{$iid} and $i_descr->{$iid} =~ m<Port\s+(\d+(?:/\d+)*)>) {
+                my $ps = $1;
+                $ps =~ s|/|.|g;
+                $if{$iid} = $ps;
+            }
+            else {
+                my $port = ( $index % $index_factor ) + $port_offset;
+                my $slot = int( $index / $index_factor ) + $slot_offset;
 
-            my $slotport = "$slot.$port";
-            $if{$iid} = $slotport;
+                my $slotport = "$slot.$port";
+                $if{$iid} = $slotport;
+            }
         }
 
     }
@@ -383,6 +396,10 @@ sub i_name {
             $i_name{$iid} = 'CPU Virtual Management IP';
         }
 
+        elsif ( ( $iid == 64 ) and ( $model =~ /^VSP[478]/ ) ) {
+            $i_name{$iid} = 'Mgmt Port';
+        }
+
         elsif ( ( $iid == 192 ) and ( $model =~ /^8[86]03/ ) ) {
             $i_name{$iid} = 'CPU 3 Ethernet Port';
         }
@@ -573,13 +590,17 @@ sub index_factor {
     # Older Accelar models use base 16 instead of 64
     $index_factor = 16
         if ( defined $model and $model =~ /^1[012][05]0/ );
-    # Newer VSP 4K uses 192?
-    $index_factor = 192
-        if ( defined $model and $model =~ /^VSP4/ );
+
     return $index_factor;
 }
 
 sub slot_offset {
+    my $passport     = shift;
+    my $model        = $passport->model();
+    # Newer VSP 4K and 8K start at an index of 192 ~ slot 3 but really slot 1
+    return -2
+        if ( defined $model and $model =~ /^VSP[478]/ );
+
     return 0;
 }
 
