@@ -1,7 +1,7 @@
 # SNMP::Info::RapidCity
 # $Id$
 #
-# Copyright (c) 2008 Eric Miller
+# Copyright (c) 2014 Eric Miller
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -39,9 +39,15 @@ use SNMP::Info;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.34';
 
-%MIBS = ( 'RAPID-CITY' => 'rapidCity', );
+%MIBS = (
+    'RAPID-CITY' => 'rapidCity',
+    # These are distinct from RAPID-CITY but are potentially used by
+    # classes which inherit the RAPID-CITY class
+    'NORTEL-NETWORKS-RAPID-SPANNING-TREE-MIB'    => 'nnRstDot1dStpVersion',
+    'NORTEL-NETWORKS-MULTIPLE-SPANNING-TREE-MIB' => 'nnMstBrgAddress',
+);
 
 %GLOBALS = (
     'rc_serial'    => 'rcChasSerialNumber',
@@ -148,6 +154,35 @@ $VERSION = '2.06';
     'rc2k_mda_part'   => 'rc2kMdaCardPartNumber',
     'rc2k_mda_date'   => 'rc2kMdaCardDateCode',
     'rc2k_mda_dev'    => 'rc2kMdaCardDeviations',
+    
+    # From RAPID-CITY::rcMltTable
+    'rc_mlt_ports'    => 'rcMltPortMembers',
+    'rc_mlt_index'    => 'rcMltIfIndex',
+    'rc_mlt_dp'       => 'rcMltDesignatedPort',
+
+    # From RAPID-CITY::rcBridgeSpbmMacTable
+    'rc_spbm_fw_port'   => 'rcBridgeSpbmMacCPort',
+    'rc_spbm_fw_status' => 'rcBridgeSpbmMacStatus',
+    'rc_spbm_fw_vlan'   => 'rcBridgeSpbmMacCVlanId',
+
+    # From RAPID-CITY::rcStgTable
+    'stp_i_id'           => 'rcStgId',
+    'rc_stp_i_mac'       => 'rcStgBridgeAddress',
+    'rc_stp_i_time'      => 'rcStgTimeSinceTopologyChange',
+    'rc_stp_i_ntop'      => 'rcStgTopChanges',
+    'rc_stp_i_root'      => 'rcStgDesignatedRoot',
+    'rc_stp_i_root_port' => 'rcStgRootPort',
+    'rc_stp_i_priority'  => 'rcStgPriority',
+
+    # From RAPID-CITY::rcStgPortTable
+    'rc_stp_p_id'       => 'rcStgPort',
+    'stp_p_stg_id'      => 'rcStgPortStgId',
+    'rc_stp_p_priority' => 'rcStgPortPriority',
+    'rc_stp_p_state'    => 'rcStgPortState',
+    'rc_stp_p_cost'     => 'rcStgPortPathCost',
+    'rc_stp_p_root'     => 'rcStgPortDesignatedRoot',
+    'rc_stp_p_bridge'   => 'rcStgPortDesignatedBridge',
+    'rc_stp_p_port'     => 'rcStgPortDesignatedPort',
 );
 
 %MUNGE = (
@@ -156,6 +191,12 @@ $VERSION = '2.06';
     'rc_cpu_mac'      => \&SNMP::Info::munge_mac,
     'rc_vlan_members' => \&SNMP::Info::munge_port_list,
     'rc_vlan_no_join' => \&SNMP::Info::munge_port_list,
+    'rc_mlt_ports'    => \&SNMP::Info::munge_port_list,
+    'rc_stp_i_mac'       => \&SNMP::Info::munge_mac,
+    'rc_stp_i_root'      => \&SNMP::Info::munge_prio_mac,
+    'rc_stp_p_root'      => \&SNMP::Info::munge_prio_mac,
+    'rc_stp_p_bridge'    => \&SNMP::Info::munge_prio_mac,
+    'rc_stp_p_port'      => \&SNMP::Info::munge_prio_port,
 );
 
 # Need to override here since overridden in Layer2 and Layer3 classes
@@ -322,6 +363,71 @@ sub i_vlan_membership {
     return $i_vlan_membership;
 }
 
+sub i_vlan_membership_untagged {
+    my $rapidcity = shift;
+
+    # Traditionally access ports have one VLAN untagged and trunk ports have
+    # one or more VLANs all tagged
+    # Newer VOSS device trunks have PerformTagging true or false and also can
+    # UntagDefaultVlan
+    # Newer BOSS device trunks have four PerformTagging options true, false,
+    # tagPvidOnly, and untagPvidOnly
+
+    my $p_members = $rapidcity->i_vlan_membership();
+    my $i_vlan = $rapidcity->i_vlan();
+    my $p_tag_opt = $rapidcity->rcVlanPortPerformTagging() || {};
+    my $p_untag_def = $rapidcity->rcVlanPortUntagDefaultVlan() || {};
+    my $p_type = $rapidcity->rcVlanPortType() || {};
+
+    my $members_untagged = {};
+    foreach my $port ( keys %$p_type ) {
+        my $type = $p_type->{$port};
+        next unless $type;
+
+        # Easiest case first access ports
+        if ($type eq 'access') {
+            # Access ports should only have one VLAN and it should be
+            # untagged
+            $members_untagged->{$port} = $p_members->{$port};
+        }
+        else {
+            # If PerformTagging has a value do all checks otherwise we're
+            # just a trunk and everything is tagged
+            if ($p_tag_opt->{$port}) {
+                if ($p_tag_opt->{$port} eq 'untagPvidOnly') {
+                    my $vlan = $i_vlan->{$port};
+                    push( @{ $members_untagged->{$port} }, $vlan );                    
+                }
+                elsif (($p_tag_opt->{$port} eq 'true') and
+                       ($p_untag_def->{$port} and $p_untag_def->{$port} eq 'true'))
+                {
+                    my $vlan = $i_vlan->{$port};
+                    push( @{ $members_untagged->{$port} }, $vlan );                     
+                }
+                elsif ($p_tag_opt->{$port} eq 'tagPvidOnly') {
+                    my $vlan = $i_vlan->{$port};
+                    my @arr = $p_members->{$port};
+                    my $index = 0;
+                    my $count = scalar @arr;
+                    $index++ until $arr[$index] eq $vlan or $index==$count;
+                    splice(@arr, $index, 1);
+                    $members_untagged->{$port} = @arr;
+                }
+                # Don't know if this is a legal configuration, but included
+                # for completeness
+                elsif ($p_tag_opt->{$port} eq 'false') {
+                    $members_untagged->{$port} = $p_members->{$port};
+                }
+                else {
+                    next;
+                }
+            }
+        }
+    }
+ 
+    return $members_untagged;
+}
+
 sub set_i_pvid {
     my $rapidcity = shift;
     my ( $vlan_id, $ifindex ) = @_;
@@ -566,13 +672,210 @@ sub _validate_vlan_param {
     return 1;
 }
 
+sub agg_ports {
+    my $rapidcity = shift;
+
+    # TODO: implement partial
+    my $ports  = $rapidcity->rc_mlt_ports;
+    my $trunks = $rapidcity->rc_mlt_index;
+    my $dps    = $rapidcity->rc_mlt_dp || {};
+
+    return {}
+        unless ref {} eq ref $trunks
+            and scalar keys %$trunks
+            and ref {} eq ref $ports
+            and scalar keys %$ports;
+
+    my $ret = {};
+    foreach my $m ( keys %$trunks ) {
+        my $idx = $trunks->{$m};
+        next unless $idx;
+        $idx = $dps->{$m} ? $dps->{$m} : $idx;
+        my $portlist = $ports->{$m};
+        next unless $portlist;
+        for ( my $i = 0; $i <= scalar(@$portlist); $i++ ) {
+            $ret->{$i} = $idx if ( @$portlist[$i] );
+        }
+    }
+
+    return $ret;
+}
+
+# break up the rcBridgeSpbmMacEntry INDEX into ISID and MAC Address.
+sub _spbm_fdbtable_index {
+    my $idx    = shift;
+    my @values = split( /\./, $idx );
+    my $isid = shift(@values);
+    return ( $isid, join( ':', map { sprintf "%02x", $_ } @values ) );
+}
+
+sub rc_spbm_fw_mac {
+    my $rapidcity  = shift;
+    my $partial = shift;
+
+    my $spbm_fw_ports = $rapidcity->rc_spbm_fw_port($partial);
+    my $spbm_fw_mac  = {};
+    foreach my $idx ( keys %$spbm_fw_ports ) {
+        my ( $isid, $mac ) = _spbm_fdbtable_index($idx);
+        $spbm_fw_mac->{$idx} = $mac;
+    }
+    return $spbm_fw_mac;
+}
+
+sub rc_spbm_fw_isid {
+    my $rapidcity  = shift;
+    my $partial = shift;
+
+    my $spbm_fw_ports = $rapidcity->rc_spbm_fw_port($partial);
+    my $spbm_fw_isid  = {};
+    foreach my $idx ( keys %$spbm_fw_ports ) {
+        my ( $isid, $mac ) = _spbm_fdbtable_index($idx);
+        $spbm_fw_isid->{$idx} = $isid;
+    }
+    return $spbm_fw_isid;
+}
+
+sub stp_ver {
+    my $rapidcity = shift;
+
+    return $rapidcity->rcSysSpanningTreeOperMode()
+      || $rapidcity->SUPER::stp_ver();
+}
+
+# RSTP and ieee8021d operating modes do not populate RAPID-CITY::rcStgTable
+# or RAPID-CITY::rcStgPortTable but do populate BRIDGE-MIB so check both
+sub stp_i_mac {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_i_mac()
+      || $rapidcity->SUPER::stp_i_mac();
+}
+
+sub stp_i_time {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_i_time()
+      || $rapidcity->SUPER::stp_i_time();
+}
+
+sub stp_i_ntop {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_i_ntop()
+      || $rapidcity->SUPER::stp_i_ntop();
+}
+
+sub stp_i_root {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_i_root()
+      || $rapidcity->SUPER::stp_i_root();
+}
+
+sub stp_i_root_port {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_i_root_port()
+      || $rapidcity->SUPER::stp_i_root_port();
+}
+
+sub stp_i_priority {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_i_priority()
+      || $rapidcity->SUPER::stp_i_priority();
+}
+
+sub stp_p_id {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_p_id()
+      || $rapidcity->SUPER::stp_p_id();
+}
+
+sub stp_p_priority {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_p_priority()
+      || $rapidcity->SUPER::stp_p_priority();
+}
+
+sub stp_p_state {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_p_state()
+      || $rapidcity->SUPER::stp_p_state();
+}
+
+sub stp_p_cost {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_p_cost()
+      || $rapidcity->SUPER::stp_p_cost();
+}
+
+sub stp_p_root {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_p_root()
+      || $rapidcity->SUPER::stp_p_root();
+}
+
+sub stp_p_bridge {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_p_bridge()
+      || $rapidcity->SUPER::stp_p_bridge();
+}
+
+sub stp_p_port {
+    my $rapidcity = shift;
+
+    return $rapidcity->rc_stp_p_port()
+      || $rapidcity->SUPER::stp_p_port();
+}
+
+sub mst_vlan2instance {
+    my $rapidcity = shift;
+    my $partial   = shift;
+
+    return $rapidcity->rcVlanStgId($partial);
+}
+
+sub i_bpduguard_enabled {
+    my $rapidcity    = shift;
+    my $partial = shift;
+
+    return $rapidcity->rcPortBpduFilteringOperEnabled();
+}
+
+sub i_stp_state {
+    my $rapidcity = shift;
+    my $partial   = shift;
+
+    my $bp_index    = $rapidcity->bp_index($partial);
+    my $stp_p_state = $rapidcity->dot1dStpPortState($partial);
+
+    my %i_stp_state;
+
+    foreach my $index ( keys %$stp_p_state ) {
+        my $state = $stp_p_state->{$index};
+        my $iid   = $bp_index->{$index};
+        next unless defined $iid;
+        next unless defined $state;
+        $i_stp_state{$iid} = $state;
+    }
+
+    return \%i_stp_state;
+}
+
 1;
 
 __END__
 
 =head1 NAME
 
-SNMP::Info::RapidCity - SNMP Interface to the Nortel RapidCity MIB
+SNMP::Info::RapidCity - SNMP Interface to the Avaya/Nortel RapidCity MIB
 
 =head1 AUTHOR
 
@@ -597,9 +900,9 @@ Eric Miller
 =head1 DESCRIPTION
 
 SNMP::Info::RapidCity is a subclass of SNMP::Info that provides an interface
-to the C<RAPID-CITY> MIB.  This MIB is used across the Nortel Ethernet Routing
-Switch and Ethernet Switch product lines (Formerly known as Passport,
-BayStack, and Accelar).
+to the C<RAPID-CITY> MIB.  This MIB is used across the Avaya/Nortel Ethernet
+Routing Switch and Ethernet Switch product lines (Formerly known as Passport,
+BayStack, and Accelar), as well as, the VSP 9000 and 7000 series.
 
 Use or create in a subclass of SNMP::Info.  Do not use directly.
 
@@ -675,6 +978,14 @@ These are methods that return scalar values from SNMP
 
 Returns serial number of the chassis
 
+=item $rapidcity->stp_ver()
+
+Returns the particular STP version running on this device.  
+
+Values: C<nortelStpg>, C<pvst>, C<rstp>, C<mstp>, C<ieee8021d>
+
+(C<rcSysSpanningTreeOperMode>)
+
 =back
 
 =head1 TABLE METHODS
@@ -711,12 +1022,43 @@ IDs.  These are the VLANs which are members of the egress list for the port.
     print "Port: $port VLAN: $vlan\n";
   }
 
+=item $rapidcity->i_vlan_membership_untagged()
+
+Returns reference to hash of arrays: key = C<ifIndex>, value = array of VLAN
+IDs.  These are the VLANs which are members of the untagged egress list for
+the port.
+
 =item $rapidcity->v_index()
 
 Returns VLAN IDs
 
 (C<rcVlanId>)
 
+=item $rapidcity->agg_ports()
+
+Returns a HASH reference mapping from slave to master port for each member of
+a port bundle (MLT) on the device. Keys are ifIndex of the slave ports,
+Values are ifIndex of the corresponding master ports.
+
+=item $rapidcity->i_stp_state()
+
+Returns the mapping of (C<dot1dStpPortState>) to the interface
+index (iid).
+
+=item $rapidcity->mst_vlan2instance()
+
+Returns the mapping of VLAN to Spanning Tree Group (STG) instance in the
+form of a hash reference with key = VLAN id, value = STG instance
+
+(C<rcVlanStgId>)
+
+=item $rapidcity->i_bpduguard_enabled()
+
+Returns true or false depending on whether C<BpduGuard> is enabled on a given
+port.  Format is a hash reference with key = C<ifIndex>, value = [true|false]
+
+(C<rcPortBpduFilteringOperEnabled>)
+
 =back
 
 =head2 RAPID-CITY Port Table (C<rcPortTable>)
@@ -1049,6 +1391,126 @@ Returns VLAN IDs
 
 =back
 
+=head2 RAPID-CITY Bridge SPBM MAC Table (C<rcBridgeSpbmMacTable>)
+
+=over
+
+=item $rapidcity->rc_spbm_fw_mac()
+
+Returns reference to hash of forwarding table MAC Addresses
+
+(C<rcBridgeSpbmMacAddr>)
+
+=item $rapidcity->rc_spbm_fw_port()
+
+Returns reference to hash of forwarding table entries port interface
+identifier (iid)
+
+(C<rcBridgeSpbmMacCPort>)
+
+=item $rapidcity->rc_spbm_fw_status()
+
+Returns reference to hash of forwarding table entries status
+
+(C<rcBridgeSpbmMacStatus>)
+
+=item $rapidcity->rc_spbm_fw_vlan()
+
+Returns reference to hash of forwarding table entries Customer VLAN ID
+
+(C<rcBridgeSpbmMacCVlanId>)
+
+=item $rapidcity->rc_spbm_fw_isid()
+
+Returns reference to hash of forwarding table entries ISID
+
+(C<rcBridgeSpbmMacIsid>)
+
+=back
+
+=head2 Spanning Tree Instance Globals
+
+C<RSTP> and C<ieee8021d> operating modes do not populate the
+C<RAPID-CITY::rcStgTable> but do populate F<BRIDGE-MIB>.  These methods check
+F<RAPID-CITY> first and fall back to F<BRIDGE-MIB>.
+
+=over
+
+=item $rapidcity->stp_i_mac()
+
+Returns the bridge address
+
+=item $rapidcity->stp_i_time()
+
+Returns time since last topology change detected. (100ths/second)
+
+=item $rapidcity->stp_i_ntop()
+
+Returns the total number of topology changes detected.
+
+=item $rapidcity->stp_i_root()
+
+Returns root of STP.
+
+=item $rapidcity->stp_i_root_port()
+
+Returns the port number of the port that offers the lowest cost path
+to the root bridge.
+
+=item $rapidcity->stp_i_priority()
+
+Returns the port number of the port that offers the lowest cost path
+to the root bridge.
+
+=back
+
+=head2 Spanning Tree Protocol Port Table
+
+C<RSTP> and C<ieee8021d> operating modes do not populate the
+C<RAPID-CITY::rcStgPortTable> but do populate F<BRIDGE-MIB>.  These methods
+check F<RAPID-CITY> first and fall back to F<BRIDGE-MIB>.
+
+=over
+
+=item $rapidcity->stp_p_id()
+
+"The port number of the port for which this entry contains Spanning Tree
+Protocol management information."
+
+=item $rapidcity->stp_p_priority()
+
+"The value of the priority field which is contained in the first
+(in network byte order) octet of the (2 octet long) Port ID.  The other octet
+of the Port ID is given by the value of C<dot1dStpPort>."
+
+=item $rapidcity->stp_p_state()
+
+"The port's current state as defined by application of the Spanning Tree
+Protocol."  
+
+=item $rapidcity->stp_p_cost()
+
+"The contribution of this port to the path cost of paths towards the spanning
+tree root which include this port."
+
+=item $rapidcity->stp_p_root()
+
+"The unique Bridge Identifier of the Bridge recorded as the Root in the
+Configuration BPDUs transmitted by the Designated Bridge for the segment to
+which the port is attached."
+
+=item $rapidcity->stp_p_bridge()
+
+"The Bridge Identifier of the bridge which this port considers to be the
+Designated Bridge for this port's segment."
+
+=item $rapidcity->stp_p_port()
+
+"The Port Identifier of the port on the Designated Bridge for this port's
+segment."
+
+=back
+
 =head1 SET METHODS
 
 These are methods that provide SNMP set functionality for overridden methods
